@@ -221,7 +221,7 @@ read_thread (void *parameter)
   g_io_channel_ref ((GIOChannel *)channel);
 
   if (channel->debug)
-    g_print ("read_thread %#x: start fd:%d, data_avail:%#x, space_avail:%#x\n",
+    g_print ("read_thread %#x: start fd:%d, data_avail:%#x space_avail:%#x\n",
 	     channel->thread_id,
 	     channel->fd,
 	     (guint) channel->data_avail_event,
@@ -334,7 +334,7 @@ create_thread (GIOWin32Channel     *channel,
 			     &channel->thread_id);
   if (channel->thread_handle == 0)
     g_warning (G_STRLOC ": Error creating reader thread: %s",
-	       strerror (errno));
+	       g_strerror (errno));
   WaitForSingleObject (channel->space_avail_event, INFINITE);
 }
 
@@ -419,7 +419,7 @@ select_thread (void *parameter)
   g_io_channel_ref ((GIOChannel *)channel);
 
   if (channel->debug)
-    g_print ("select_thread %#x: start fd:%d,\n\tdata_avail:%#x, data_avail_noticed:%#x\n",
+    g_print ("select_thread %#x: start fd:%d data_avail:%#x data_avail_noticed:%#x\n",
 	     channel->thread_id,
 	     channel->fd,
 	     (guint) channel->data_avail_event,
@@ -482,8 +482,7 @@ select_thread (void *parameter)
 	channel->revents |= G_IO_ERR;
 
       if (channel->debug)
-	g_print ("select_thread %#x: resetting data_avail_noticed,\n"
-		 "\tsetting data_avail\n",
+	g_print ("select_thread %#x: resetting data_avail_noticed, setting data_avail\n",
 		 channel->thread_id);
       ResetEvent (channel->data_avail_noticed_event);
       SetEvent (channel->data_avail_event);
@@ -537,15 +536,27 @@ g_io_win32_prepare (GSource *source,
 		    gint    *timeout)
 {
   GIOWin32Watch *watch = (GIOWin32Watch *)source;
+  GIOCondition buffer_condition = g_io_channel_get_buffer_condition (watch->channel);
   GIOWin32Channel *channel = (GIOWin32Channel *)watch->channel;
   
   *timeout = -1;
   
+  if (channel->debug)
+    g_print ("g_io_win32_prepare: for thread %#x buffer_condition:%#x\n"
+	     "  watch->pollfd.events:%#x watch->pollfd.revents:%#x channel->revents:%#x\n",
+	     channel->thread_id, buffer_condition,
+	     watch->pollfd.events, watch->pollfd.revents, channel->revents);
+
   if (channel->type == G_IO_WIN32_FILE_DESC)
     {
       LOCK (channel->mutex);
       if (channel->running && channel->wrp == channel->rdp)
-	channel->revents = 0;
+	{
+	  if (channel->debug)
+	    g_print ("g_io_win32_prepare: for thread %#x, setting channel->revents = 0\n",
+		     channel->thread_id);
+	  channel->revents = 0;
+	}
       UNLOCK (channel->mutex);
     }
   else if (channel->type == G_IO_WIN32_SOCKET)
@@ -553,7 +564,7 @@ g_io_win32_prepare (GSource *source,
       channel->revents = 0;
 
       if (channel->debug)
-	g_print ("g_io_win32_prepare: thread %#x, setting data_avail_noticed\n",
+	g_print ("g_io_win32_prepare: for thread %#x, setting data_avail_noticed\n",
 		 channel->thread_id);
       SetEvent (channel->data_avail_noticed_event);
       if (channel->debug)
@@ -561,11 +572,7 @@ g_io_win32_prepare (GSource *source,
 		 channel->thread_id);
     }
 
-  return FALSE;
-  /* XXX: why should we want to do this ? */
-  watch->condition = g_io_channel_get_buffer_condition (watch->channel);
-
-  return (watch->pollfd.revents & (G_IO_IN | G_IO_OUT)) == watch->condition;
+  return ((watch->condition & buffer_condition) == watch->condition);
 }
 
 static gboolean
@@ -577,9 +584,9 @@ g_io_win32_check (GSource *source)
   GIOCondition buffer_condition = g_io_channel_get_buffer_condition (watch->channel);
 
   if (channel->debug)
-    g_print ("g_io_win32_check: for thread %#x:\n"
-	     "\twatch->pollfd.events:%#x, watch->pollfd.revents:%#x, channel->revents:%#x\n",
-	     channel->thread_id,
+    g_print ("g_io_win32_check: for thread %#x buffer_condition:%#x\n"
+	     "  watch->pollfd.events:%#x watch->pollfd.revents:%#x channel->revents:%#x\n",
+	     channel->thread_id, buffer_condition,
 	     watch->pollfd.events, watch->pollfd.revents, channel->revents);
 
   if (channel->type != G_IO_WIN32_WINDOWS_MESSAGES)
@@ -602,7 +609,7 @@ g_io_win32_check (GSource *source)
 		 channel->thread_id);
     }
 
-  return (watch->pollfd.revents & watch->condition);
+  return ((watch->pollfd.revents | buffer_condition) & watch->condition);
 }
 
 static gboolean
@@ -612,6 +619,7 @@ g_io_win32_dispatch (GSource     *source,
 {
   GIOFunc func = (GIOFunc)callback;
   GIOWin32Watch *watch = (GIOWin32Watch *)source;
+  GIOCondition buffer_condition = g_io_channel_get_buffer_condition (watch->channel);
   
   if (!func)
     {
@@ -621,7 +629,7 @@ g_io_win32_dispatch (GSource     *source,
     }
   
   return (*func) (watch->channel,
-		  watch->pollfd.revents & watch->condition,
+		  (watch->pollfd.revents | buffer_condition) & watch->condition,
 		  user_data);
 }
 
@@ -852,7 +860,7 @@ g_io_win32_fd_read (GIOChannel *channel,
           default:
             g_set_error (err, G_IO_CHANNEL_ERROR,
                          g_io_channel_error_from_errno (errno),
-                         strerror (errno));
+                         g_strerror (errno));
             return G_IO_STATUS_ERROR;
         }
     }
@@ -890,7 +898,7 @@ g_io_win32_fd_write (GIOChannel  *channel,
           default:
             g_set_error (err, G_IO_CHANNEL_ERROR,
                          g_io_channel_error_from_errno (errno),
-                         strerror (errno));
+                         g_strerror (errno));
             return G_IO_STATUS_ERROR;
         }
     }
@@ -932,7 +940,7 @@ g_io_win32_fd_seek (GIOChannel *channel,
     {
       g_set_error (err, G_IO_CHANNEL_ERROR,
 		   g_io_channel_error_from_errno (EINVAL),
-		   strerror (EINVAL));
+		   g_strerror (EINVAL));
       return G_IO_STATUS_ERROR;
     }
   
@@ -942,7 +950,7 @@ g_io_win32_fd_seek (GIOChannel *channel,
     {
       g_set_error (err, G_IO_CHANNEL_ERROR,
 		   g_io_channel_error_from_errno (errno),
-		   strerror (errno));
+		   g_strerror (errno));
       return G_IO_STATUS_ERROR;
     }
 
@@ -1236,7 +1244,7 @@ g_io_channel_new_file (const gchar  *filename,
     {
       g_set_error (error, G_FILE_ERROR,
                    g_file_error_from_errno (errno),
-                   strerror (errno));
+                   g_strerror (errno));
       return (GIOChannel *)NULL;
     }
 
