@@ -33,7 +33,6 @@
  */
 
 #include "config.h"
-#include "glib.h"
 
 #ifdef G_THREAD_USE_PID_SURROGATE
 #include <sys/types.h>
@@ -48,18 +47,26 @@
 
 #include <string.h>
 
+#include "glib.h"
+
+
 #if GLIB_SIZEOF_SYSTEM_THREAD == SIZEOF_VOID_P
-# define g_system_thread_equal(thread1, thread2) 			\
-   (thread1.dummy_pointer == thread2.dummy_pointer)
+# define g_system_thread_equal_simple(thread1, thread2)			\
+   ((thread1).dummy_pointer == (thread2).dummy_pointer)
 # define g_system_thread_assign(dest, src)				\
-   (dest.dummy_pointer = src.dummy_pointer)
+   ((dest).dummy_pointer = (src).dummy_pointer)
 #else /* GLIB_SIZEOF_SYSTEM_THREAD != SIZEOF_VOID_P */
-# define g_system_thread_equal(thread1, thread2)			\
-   (memcmp (&thread1, &thread2, GLIB_SIZEOF_SYSTEM_THREAD) == 0)
+# define g_system_thread_equal_simple(thread1, thread2)			\
+   (memcmp (&(thread1), &(thread2), GLIB_SIZEOF_SYSTEM_THREAD) == 0)
 # define g_system_thread_assign(dest, src)				\
-   (memcpy (&dest, &src, GLIB_SIZEOF_SYSTEM_THREAD))
+   (memcpy (&(dest), &(src), GLIB_SIZEOF_SYSTEM_THREAD))
 #endif /* GLIB_SIZEOF_SYSTEM_THREAD == SIZEOF_VOID_P */
 
+#define g_system_thread_equal(thread1, thread2)				\
+  (g_thread_functions_for_glib_use.thread_equal ? 			\
+   g_thread_functions_for_glib_use.thread_equal (&(thread1), &(thread2)) :\
+   g_system_thread_equal_simple((thread1), (thread2)))
+
 GQuark 
 g_thread_error_quark (void)
 {
@@ -83,7 +90,7 @@ struct  _GRealThread
 };
 
 #ifdef G_THREAD_USE_PID_SURROGATE
-static gint priority_map[] = { 15, 0, -15, -20 };
+static gint priority_map[4];
 static gboolean prio_warned = FALSE;
 # define SET_PRIO(pid, prio) G_STMT_START{				\
   gint error = setpriority (PRIO_PROCESS, (pid), priority_map[prio]);	\
@@ -166,6 +173,17 @@ g_mutex_init (void)
   G_THREAD_UF (thread_self, (&main_thread->system_thread));
 
   g_mutex_protect_static_mutex_allocation = g_mutex_new ();
+
+#ifdef G_THREAD_USE_PID_SURROGATE
+  priority_map[G_THREAD_PRIORITY_NORMAL] = 
+    getpriority (PRIO_PROCESS, (getpid ()));
+  priority_map[G_THREAD_PRIORITY_LOW] = 
+    MIN (20, priority_map[G_THREAD_PRIORITY_NORMAL] + 10);
+  priority_map[G_THREAD_PRIORITY_HIGH] = 
+    MAX (-20, priority_map[G_THREAD_PRIORITY_NORMAL] - 10);
+  priority_map[G_THREAD_PRIORITY_URGENT] = 
+    MAX (-20, priority_map[G_THREAD_PRIORITY_NORMAL] - 15);
+#endif /* G_THREAD_USE_PID_SURROGATE */
 }
 
 void 
@@ -313,7 +331,7 @@ g_static_rec_mutex_lock_full   (GStaticRecMutex *mutex,
 guint    
 g_static_rec_mutex_unlock_full (GStaticRecMutex *mutex)
 {
-  gint depth;
+  guint depth;
 
   g_return_val_if_fail (mutex, 0);
 
@@ -544,12 +562,14 @@ g_thread_create_full (GThreadFunc 		 func,
 		      GThreadPriority 	 priority,
 		      GError                **error)
 {
-  GRealThread* result = g_new (GRealThread, 1);
+  GRealThread* result;
   GError *local_error = NULL;
   g_return_val_if_fail (func, NULL);
   g_return_val_if_fail (priority >= G_THREAD_PRIORITY_LOW, NULL);
   g_return_val_if_fail (priority <= G_THREAD_PRIORITY_URGENT, NULL);
   
+  result = g_new (GRealThread, 1);
+
   result->thread.joinable = joinable;
   result->thread.priority = priority;
   result->thread.func = func;
