@@ -257,6 +257,23 @@ g_param_spec_get_blurb (GParamSpec *pspec)
   return pspec->_blurb;
 }
 
+static void
+canonicalize_key (gchar *key)
+{
+  gchar *p;
+  
+  for (p = key; *p != 0; p++)
+    {
+      gchar c = *p;
+      
+      if (c != '-' &&
+	  (c < '0' || c > '9') &&
+	  (c < 'A' || c > 'Z') &&
+	  (c < 'a' || c > 'z'))
+	*p = '-';
+    }
+}
+
 gpointer
 g_param_spec_internal (GType        param_type,
 		       const gchar *name,
@@ -265,18 +282,18 @@ g_param_spec_internal (GType        param_type,
 		       GParamFlags  flags)
 {
   GParamSpec *pspec;
-
+  
   g_return_val_if_fail (G_TYPE_IS_PARAM (param_type) && param_type != G_TYPE_PARAM, NULL);
   g_return_val_if_fail (name != NULL, NULL);
   g_return_val_if_fail ((name[0] >= 'A' && name[0] <= 'Z') || (name[0] >= 'a' && name[0] <= 'z'), NULL);
-
+  
   pspec = (gpointer) g_type_create_instance (param_type);
   pspec->name = g_strdup (name);
-  g_strcanon (pspec->name, G_CSET_A_2_Z G_CSET_a_2_z G_CSET_DIGITS "-", '-');
+  canonicalize_key (pspec->name);
   pspec->_nick = g_strdup (nick);
   pspec->_blurb = g_strdup (blurb);
   pspec->flags = (flags & G_PARAM_USER_MASK) | (flags & G_PARAM_MASK);
-
+  
   return pspec;
 }
 
@@ -650,7 +667,7 @@ param_spec_ht_lookup (GHashTable  *hash_table,
       key.name = g_strdup (param_name);
       key.owner_type = owner_type;
       
-      g_strcanon (key.name, G_CSET_A_2_Z G_CSET_a_2_z G_CSET_DIGITS "-", '-');
+      canonicalize_key (key.name);
       if (walk_ancestors)
 	do
 	  {
